import { CreateResult, Draft, Options, PatchesOptions, Result } from './interface';
/**
 * `create(baseState, callback, options)` to create the next state
 *
 * ## Example
 *
 * ```ts
 * import { create } from '../index';
 *
 * const baseState = { foo: { bar: 'str' }, arr: [] };
 * const state = create(
 *   baseState,
 *   (draft) => {
 *     draft.foo.bar = 'str2';
 *   },
 * );
 *
 * expect(state).toEqual({ foo: { bar: 'str2' }, arr: [] });
 * expect(state).not.toBe(baseState);
 * expect(state.foo).not.toBe(baseState.foo);
 * expect(state.arr).toBe(baseState.arr);
 * ```
 */
declare function create<T extends any, F extends boolean = false, O extends PatchesOptions = false, R extends void | Promise<void> | T | Promise<T> = void>(base: T, mutate: (draft: Draft<T>) => R, options?: Options<O, F>): CreateResult<T, O, F, R>;
declare function create<T extends any, F extends boolean = false, O extends PatchesOptions = false, R extends void | Promise<void> = void>(base: T, mutate: (draft: T) => R, options?: Options<O, F>): CreateResult<T, O, F, R>;
declare function create<T extends any, P extends any[] = [], F extends boolean = false, O extends PatchesOptions = false, R extends void | Promise<void> = void>(mutate: (draft: Draft<T>, ...args: P) => R, options?: Options<O, F>): (base: T, ...args: P) => CreateResult<T, O, F, R>;
declare function create<T extends any, O extends PatchesOptions = false, F extends boolean = false>(base: T, options?: Options<O, F>): [T, () => Result<T, O, F>];
export { create };
